describe "an instance generated by a factory" do
  before do
    define_model("User")

    FactoryBot.define do
      factory :user
    end
  end

  it "registers the user factory" do
    expect(FactoryBot::Internal.factory_by_name(:user))
      .to be_a(FactoryBot::Factory)
  end
end
